//+------------------------------------------------------------------+
//|                                                        frama.mq4 |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1  clrDeepSkyBlue
#property indicator_color2  clrSandyBrown
#property indicator_width1  2
#property indicator_width2  2
#property indicator_minimum 0
#property indicator_maximum 1
#property strict

//
//
//
//
//

enum enFilterWhat
{
   flt_prc,  // Filter the price
   flt_pfr,  // Filter the price and frama
   flt_pfs,  // Filter the price and signal
   flt_frm,  // Filter the frama
   flt_frs,  // Filter the frama and signal
   flt_sig,  // Filter the signal
   flt_all   // Filter all
};

extern ENUM_TIMEFRAMES    TimeFrame        = PERIOD_CURRENT; // Time frame to use
extern int                FramaPeriod      = 25;             // Frama period
extern ENUM_APPLIED_PRICE FramaPrice       = PRICE_CLOSE;    // Price to use
extern double             FramaMultiplier  = 4.6;            // Frama multiplier
extern double             SignalMultiplier = 2.5;            // Signal multiplier
extern double             Filter           = 0;              // Filter to use for filtering (<=0 for no filtering)
extern int                FilterPeriod     = 0;              // Filter period (0<= use indicator main period)
extern enFilterWhat       FilterOn         = flt_all;        // Apply filter to :
extern bool               alertsOn         = false;          // Turn alerts on
extern bool               alertsOnCurrent  = false;          // Alerts on current (still opened) bar?
extern bool               alertsMessage    = true;           // Alerts should display a pop-up message?
extern bool               alertsSound      = true;           // Alerts should play an alert sound?
extern bool               alertsEmail      = false;          // Alerts should send an email?
extern bool               alertsNotify     = false;          // Alerts should send push notification?

double frmBuffer[],frsBuffer[],histou[],histod[],state[];
string indicatorFileName;
bool   returnBars;

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(5);
   SetIndexBuffer(0,histou); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,histod); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,frmBuffer);
   SetIndexBuffer(3,frsBuffer);
   SetIndexBuffer(4,state);
      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame==-99;
      TimeFrame         = MathMax(TimeFrame,_Period);
   IndicatorShortName("Frama ribbon ("+(string)FramaPeriod+")");         
   return(0);
}

int deinit()
{
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars < 0)   return(-1);
      if(counted_bars > 0)   counted_bars--;
            int limit = MathMin(Bars-counted_bars,Bars-1);
            if (returnBars) { histou[0] = limit+1; return(0); }
            if (TimeFrame != Period())
            {
               limit = (int)MathMax(limit,MathMin(Bars-1,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/_Period));
               for(int i = limit; i >= 0; i--)
               {
                  int y = iBarShift(NULL,TimeFrame,Time[i]);
                  state[i]  = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,FramaPeriod,FramaPrice,FramaMultiplier,SignalMultiplier,Filter,FilterPeriod,FilterOn,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,alertsNotify,4,y); 
                  histou[i] = EMPTY_VALUE;
                  histod[i] = EMPTY_VALUE;
                  if (state[i]== 1) histou[i] = 1;
                  if (state[i]==-1) histod[i] = 1;
               }
               return(0); 
            }

   //
   //
   //
   //
   //

      
      int    tperiod = FilterPeriod; if (tperiod<=0) tperiod = FramaPeriod;
      double pfilter = Filter, vfilter = Filter, sfilter = Filter;
      switch(FilterOn)
      {
         case flt_prc: vfilter=sfilter=0; break;
         case flt_pfr: sfilter=0;         break;
         case flt_pfs: vfilter=0;         break;
         case flt_frm: pfilter=sfilter=0; break;
         case flt_frs: pfilter=0;         break;
         case flt_sig: pfilter=vfilter=0; break;
      }         
      for(int i = limit; i >= 0; i--)
      {
         frmBuffer[i] = iFilter(iFrama(iFilter(iMA(NULL,0,1,0,MODE_SMA,FramaPrice,i),pfilter,tperiod,i,0),FramaPeriod,FramaMultiplier ,i,0),vfilter,tperiod,i,1);
         frsBuffer[i] = iFilter(iFrama(frmBuffer[i]                                                      ,FramaPeriod,SignalMultiplier,i,1),sfilter,tperiod,i,2);
         histou[i]    = EMPTY_VALUE;
         histod[i]    = EMPTY_VALUE;
            if (i<Bars-1) state[i] = state[i+1];
            if (frmBuffer[i]>frsBuffer[i]) state[i] =  1;
            if (frmBuffer[i]<frsBuffer[i]) state[i] = -1;
            if (state[i]== 1) histou[i] = 1;
            if (state[i]==-1) histod[i] = 1;
      }         
   manageAlerts();
   return(0);      
}


//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

#define filterInstances 3
double workFil[][filterInstances*3];

#define _fchange 0
#define _fachang 1
#define _fprice  2

double iFilter(double tprice, double filter, int period, int i, int instanceNo=0)
{
   if (filter<=0) return(tprice);
   if (ArrayRange(workFil,0)!= Bars) ArrayResize(workFil,Bars); i = Bars-i-1; instanceNo*=3;
   
   //
   //
   //
   //
   //
   
   workFil[i][instanceNo+_fprice]  = tprice; if (i<1) return(tprice);
   workFil[i][instanceNo+_fchange] = MathAbs(workFil[i][instanceNo+_fprice]-workFil[i-1][instanceNo+_fprice]);
   workFil[i][instanceNo+_fachang] = workFil[i][instanceNo+_fchange];

   for (int k=1; k<period && (i-k)>=0; k++) workFil[i][instanceNo+_fachang] += workFil[i-k][instanceNo+_fchange];
                                            workFil[i][instanceNo+_fachang] /= period;
    
   double stddev = 0; for (int k=0;  k<period && (i-k)>=0; k++) stddev += MathPow(workFil[i-k][instanceNo+_fchange]-workFil[i-k][instanceNo+_fachang],2);
          stddev = MathSqrt(stddev/(double)period); 
   double filtev = filter * stddev;
   if( MathAbs(workFil[i][instanceNo+_fprice]-workFil[i-1][instanceNo+_fprice]) < filtev ) workFil[i][instanceNo+_fprice]=workFil[i-1][instanceNo+_fprice];
        return(workFil[i][instanceNo+_fprice]);
}


//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      int whichBar = 1; if (alertsOnCurrent) whichBar = 0;
      if (state[whichBar] != state[whichBar+1])
      {
         if (state[whichBar]== 1) doAlert(whichBar,"trend changed to up");
         if (state[whichBar]==-1) doAlert(whichBar,"trend changed to down");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message = timeFrameToString(_Period)+" "+Symbol()+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+" frama ribbon "+doWhat;
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(Symbol()+" frama ribbon ",message);
          if (alertsNotify)  SendNotification(message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

#define framaInstances 2
double workFrama[][framaInstances];
double iFrama(double price, int fperiod, double fmultiplier, int i, int instanceNo=0)
{
   if (ArrayRange(workFrama,0)!=Bars) ArrayResize(workFrama,Bars); int r = Bars-i-1;
   
   //
   //
   //
   //
   //
   
   if (r<1)
      workFrama[r][instanceNo] = price;
   else
   {      
      double halfPeriod = fperiod/2;
      double hh = High[iHighest(NULL,0,MODE_HIGH,fperiod,i)];
      double ll = Low [iLowest (NULL,0,MODE_LOW ,fperiod,i)];
      double n3 = (hh-ll)/FramaPeriod;
             hh = High[iHighest(NULL,0,MODE_HIGH,(int)halfPeriod,i)];
             ll = Low [iLowest (NULL,0,MODE_LOW ,(int)halfPeriod,i)];
      double n1 = (hh-ll)/halfPeriod;
      int   ind = MathMin(i+(int)halfPeriod,Bars-1);
             hh = High[iHighest(NULL,0,MODE_HIGH,(int)halfPeriod,ind)];
             ll = Low [iLowest (NULL,0,MODE_LOW ,(int)halfPeriod,ind)];
      double n2 = (hh-ll)/halfPeriod;
      double dimen = 0;
            if (n1>0 && n2>0 && n3>0) dimen = (MathLog(n1+n2)-MathLog(n3))/MathLog(2.0);
      double alpha = MathMin(MathMax(MathExp(-fmultiplier*(dimen-1.0)),0.01),1.0);
      workFrama[r][instanceNo] = alpha*price+(1-alpha)*workFrama[r-1][instanceNo];
   }
   return(workFrama[r][instanceNo]);      
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}